@@ -1,5 +1,7 @@
 (in-package #:common-lisp-user)
 
+(proclaim '(optimize (speed 2) (safety 3) #-allegro (debug 3)))
+
 (load (make-pathname :name "script-support" :defaults *load-pathname*))
 
 (cond
@@ -21,6 +23,7 @@
            (compile-file *asdf-lisp* :output-file tmp :print t :verbose t))
        (declare (ignore result))
        (cond
+         #-ecl ;; 10.7.1 from CVS has spurious warnings. Sigh.
          (warnings-p
           (leave-lisp "Testsuite failed: ASDF compiled with warnings" 1))
          (errors-p
