@@ -1,5 +1,8 @@
 (in-package #:common-lisp-user)
 
+(declaim (optimize (speed 2) (safety 3) #-allegro (debug 3)))
+(proclaim '(optimize (speed 2) (safety 3) #-allegro (debug 3)))
+
 ;; We can't use asdf:merge-pathnames* because ASDF isn't loaded yet.
 ;; We still want to work despite and host/device funkiness.
 (defparameter *test-directory*
